package it.eng.spago.cms.access;

import it.eng.spago.base.SourceBean;
import it.eng.spago.cms.exceptions.ConfigurationException;
import it.eng.spago.cms.init.CMSManager;
import it.eng.spago.cms.util.constants.ModuleConstants;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.tracing.TracerSingleton;

import java.util.List;


public class PermissionVerifierManager {

	private static PermissionVerifier verifier = null;
	
	public static PermissionVerifier getPermissionVerifier() throws ConfigurationException {
		return getPermissionVerifier(CMSManager.getDefaultRepository());
	}
	
	
	public static PermissionVerifier getPermissionVerifier(String nameRep) throws ConfigurationException {
		if(verifier == null) {
			Object repConfObj  = ConfigSingleton.getInstance().getFilteredSourceBeanAttribute("CONTENTCONFIGURATION.CONTENTREPOSITORY", "name", nameRep);
		    if(repConfObj == null) {
		    	TracerSingleton.log(ModuleConstants.NAME_MODULE, TracerSingleton.CRITICAL,
	            "PermissionVerifierManager::getPermissionVerifier:configuration for repository "+nameRep+" not found");
		    	throw new ConfigurationException("PermissionVerifierManager::getPermissionVerifier:configuration not found");
		    }
		    else if(repConfObj instanceof List) {
		    	TracerSingleton.log(ModuleConstants.NAME_MODULE, TracerSingleton.CRITICAL,
		        "PermissionVerifierManager::getPermissionVerifier:exists more than one configuration for the repository "+nameRep);
		    	throw new ConfigurationException("PermissionVerifierManager::getPermissionVerifier:more than one configuration");
		    }
		    else {
		    	SourceBean repoConfSB = (SourceBean)repConfObj;
		    	String classSecurity = (String)repoConfSB.getAttribute("SECURITY.class");
		    	if((classSecurity==null) || (classSecurity.equalsIgnoreCase("")) ) {
		    		TracerSingleton.log(ModuleConstants.NAME_MODULE, TracerSingleton.WARNING,
		    		"PermissionVerifierManager::getPermissionVerifier: security class null, using default");
		    		classSecurity = "it.eng.spago.cms.access.verifiers.DefaultPermissionVerifier";
		    	}
		    	try {
		    		verifier = (PermissionVerifier)Class.forName(classSecurity).newInstance();
		    		verifier.init(nameRep);
		    	} catch (Exception e) {
		    		TracerSingleton.log(ModuleConstants.NAME_MODULE, TracerSingleton.CRITICAL,
		    		"PermissionVerifierManager::getPermissionVerifier: cannot create an instance of security class");
		    		throw new ConfigurationException("PermissionVerifierManager::getPermissionVerifier: cannot create an instance of security class");
		    	}
		    	return verifier;
		    }
		} else {
			return verifier;
		}
	}
	
}
