package it.eng.spago.cms.exec;

import it.eng.javax.jcr.Node;
import it.eng.javax.jcr.NodeIterator;
import it.eng.javax.jcr.PathNotFoundException;
import it.eng.javax.jcr.Property;
import it.eng.javax.jcr.PropertyIterator;
import it.eng.javax.jcr.PropertyType;
import it.eng.javax.jcr.RepositoryException;
import it.eng.javax.jcr.Session;
import it.eng.javax.jcr.Value;
import it.eng.javax.jcr.version.Version;
import it.eng.javax.jcr.version.VersionIterator;
import it.eng.spago.cms.access.PermissionVerifier;
import it.eng.spago.cms.access.PermissionVerifierManager;
import it.eng.spago.cms.exceptions.OperationExecutionException;
import it.eng.spago.cms.exceptions.PathNotValidException;
import it.eng.spago.cms.util.OperationDescriptorUtils;
import it.eng.spago.cms.util.Path;
import it.eng.spago.cms.util.RepositoryNodeUtils;
import it.eng.spago.cms.util.constants.OperationsConstants;
import it.eng.spago.cms.util.constants.PropertiesConstants;
import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.tracing.TracerSingleton;

import java.io.InputStream;



/**
 * 
 * Class that define methods for the get operation on the repository
 * 
 */

public class GetObjectImpl {
	
	/**
	 * Method taht perform the get operation on a node of the repository
	 * and return a SourceBean contains all request information.
	 * 	 
	 * @param parameters SourceBean operation descriptor
	 * @param conn Connection to the repository
	 * @return SourceBean contains all request information
	 * @throws EMFInternalError
	 */
	protected SourceBean execute(SourceBean parameters, CMSConnection conn, String [] roles) throws OperationExecutionException {
		
		SourceBean sourceBean = null;
		Session session = conn.getSession();
		String prefixSys = conn.getPrefixPropertySystem();
		String prefixUsr = conn.getPrefixPropertyUser();
		
        String pathStr = null;  
		String verNameStr = null;
		String verLabelStr = null;
		Boolean getVersions;
		Boolean getContent;
		Boolean getChilds;
		Boolean getProperties;
		boolean versionExist;
		boolean isContainer; 
		Node node = null;
		
		try {
	
			// get path string new object
		    pathStr = OperationDescriptorUtils.getStringAttribute(parameters, OperationsConstants.PATH);
			// get version name to retrive  (version name can be null)			
            verNameStr = OperationDescriptorUtils.getStringAttribute(parameters, OperationsConstants.VERSION);
            // get flag of required information
            getVersions = OperationDescriptorUtils.getBooleanAttribute(parameters, OperationsConstants.GETVERSIONS);
            if(getVersions == null) { getVersions = new Boolean(false); }
            getChilds = OperationDescriptorUtils.getBooleanAttribute(parameters, OperationsConstants.GETCHILDS);
            if(getChilds == null) { getChilds = new Boolean(false); }
            getProperties = OperationDescriptorUtils.getBooleanAttribute(parameters, OperationsConstants.GETPROPERTIES);
            if(getProperties == null) { getProperties = new Boolean(false); }
            getContent = OperationDescriptorUtils.getBooleanAttribute(parameters, OperationsConstants.GETCONTENT);
            if(getContent == null) { getContent = new Boolean(false); }
            // create the path object  
			Path path = Path.create(pathStr);

			
			// get node
			if(path.isRootNode()) {
				node = session.getRootNode();
				isContainer = true;
			} else {
		    	node = (Node)session.getRootNode().getNode(path.getRootRelativePathStr());
			}            
            
			
			// if version name is set control if version exist
			if( (verNameStr!= null) && !verNameStr.trim().equalsIgnoreCase("") ) {
				versionExist = RepositoryNodeUtils.versionNameExist(node, verNameStr);
				if(!versionExist) {
		    		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MAJOR,
			    	"GetObjectImpl::execute: version doesn't exists");				
				    throw new OperationExecutionException("GetObjectImpl::execute: version doesn't exists");
		    	}
			}
			
			
            // control if user has permission to read
			PermissionVerifier verifier = null;
			try {
				verifier = PermissionVerifierManager.getPermissionVerifier(conn.getNameRepositoryPool());
			} catch (Exception e) {
				throw new OperationExecutionException("GetObjectImpl::execute: error during the " +
		                "PermissionVerifier recover", e);
			}
			if(!verifier.canRead(pathStr, roles)) {
				TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
		    	"GetObjectImpl::execute: GetObjectImpl::execute: permission to read denied");
				//throw new OperationExecutionException("GetObjectImpl::execute: permission to read  " +
		        //        "denied");
				return null;
			}
			
            
            // checkout node
			if(node.isNodeType("mix:versionable")) {
				if(!node.isCheckedOut()) {
					node.checkout();
				}
			}
			
			
			
			sourceBean = toSourceBean(conn, node, verNameStr, getVersions.booleanValue(), 
									  getChilds.booleanValue(), getProperties.booleanValue(), 
									  getContent.booleanValue());
			 						
		} catch (PathNotFoundException e) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MINOR,
	    	"GetObjectImpl::execute: object not found");	
		} catch (RepositoryException e) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
	    	"GetObjectImpl::execute: repository acess error");
			throw new OperationExecutionException("GetObjectImpl::execute: repository acess error");
		} catch (PathNotValidException e) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MAJOR,
	    	"GetObjectImpl::execute: malformed path");
			throw new OperationExecutionException("GetObjectImpl::execute: malformed path");
		} 
		
		// return the result sourceBean
		return sourceBean;
	}

	
	
	
	
	/**
	 * Retrive request information from a node a store them 
	 * inside a SourceBean with a fixed structure (see 
	 * Documentaion of Spago CMS)
	 * 
	 * @param conn, connection to the repository
	 * @param node, node from which retrive information
	 * @param ver, name version to retrive
	 * @param incVer, flag to include versions informations
	 * @param incChild, flag to include childs informations
	 * @param incProp, flag to include properties informations
	 * @param incCont, flag to include Content information
	 * @return SourceBean contains all request inforamtions
	 * @throws EMFInternalError
	 */
	private SourceBean toSourceBean(CMSConnection conn, Node node, String ver, boolean incVer, 
			                        boolean incChild, boolean incProp, boolean incCont) throws OperationExecutionException {
	
		SourceBean sourceBean = null;
		String prefixSys = conn.getPrefixPropertySystem();
		String prefixUsr = conn.getPrefixPropertyUser();
		String objectType = "";
		boolean requestVer = false;
		if( (ver != null) && (!ver.trim().equals("")) ) {
			requestVer = true;
		}
		Node version = null;
				
	  	// control if node is container or content
		boolean isContainer = true;
		try {
			if(node.getProperty(( prefixSys + ":"+OperationsConstants.TYPE)).getString().equalsIgnoreCase(OperationsConstants.CONTAINER)) {
				isContainer = true;
			} else {
				isContainer = false;
			}
		}
		catch (PathNotFoundException pnfe) {}
		catch (Exception e) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MAJOR,
	    	"GetObjectImpl::toSourceBean: impossible to retrive property type of the node");				
		    throw new OperationExecutionException("GetObjectImpl::execute: impossible to retrive property type of the node");
		}
		
		try {
			sourceBean = new SourceBean(OperationsConstants.NODE);
			if(requestVer) {
				try {
					version = node.getVersionHistory().getVersion(ver).getNodes().nextNode();
				} catch (Exception e) {
					requestVer = false;
				}
			}
				
			// retrive and store name, path, base version and type of the node
			sourceBean.setAttribute(OperationsConstants.NAME, node.getName());
			sourceBean.setAttribute(OperationsConstants.PATH, node.getPath());
		    try{
		    	String nameCurVer = node.getBaseVersion().getName();
		    	sourceBean.setAttribute(OperationsConstants.VERSION, nameCurVer);
		    } catch(Exception e) {
		    	sourceBean.setAttribute(OperationsConstants.VERSION, "");
		    }
			try{
				String uuid = node.getUUID();
				sourceBean.setAttribute(OperationsConstants.UUID, uuid);
			} catch(Exception e) {
				sourceBean.setAttribute(OperationsConstants.UUID, "");
			}
			if(isContainer) {
				sourceBean.setAttribute(OperationsConstants.TYPE, OperationsConstants.CONTAINER);
			} else {
				sourceBean.setAttribute(OperationsConstants.TYPE, OperationsConstants.CONTENT);
			}
			
			
			// retrive versions
			if(incVer && !requestVer) {
				SourceBean versionsSB = new SourceBean(OperationsConstants.VERSIONS);
				if(node.isNodeType("mix:versionable")) {
					try{
						VersionIterator vit = node.getVersionHistory().getAllVersions();
						while (vit.hasNext()){
							SourceBean versionSB = new SourceBean(OperationsConstants.VERSION);
							Version v = (Version)vit.next();
							if(!v.getName().equalsIgnoreCase("jcr:rootVersion")) {
								versionSB.setAttribute(OperationsConstants.NAME, v.getName());
								versionSB.setAttribute(OperationsConstants.DATA_CREATION, v.getCreated().getTime().toString());
								versionsSB.setAttribute(versionSB);
							}
						}
					} catch (Exception e) {
						// ignore
					}
				}
				sourceBean.setAttribute(versionsSB);
			}
			
			
			
			
			// get properties
			if(incProp) {
				Node nodeForProp = node;
				if(requestVer){
					nodeForProp = version;
				}
				
				PropertyIterator properties = nodeForProp.getProperties(prefixUsr + ":*");
				SourceBean propertiesSB = new SourceBean(PropertiesConstants.PROPERTIES);
				while (properties.hasNext()){
					Property property = (Property)properties.next();				
					SourceBean propertySB = new SourceBean(PropertiesConstants.PROPERTY);
					// get property name
					String name = property.getName();
					name = name.substring(prefixUsr.length()+1);
					// get values
					Value[] values = null;
					try {
						values = property.getValues();
					} catch (Exception e) {
						values = new Value[1];
						values[0] = property.getValue();
					}
					// get code type of the property
					int codType = values[0].getType();
					// get name type of the property
					String type = PropertyType.nameFromValue(codType);	
					propertySB.setAttribute(PropertiesConstants.NAME_PROPERTY, name);
					propertySB.setAttribute(PropertiesConstants.CODE_TYPE_PROPERTY, String.valueOf(codType));
					for(int i=0; i<values.length; i++) {
						SourceBean valueSB = new SourceBean(PropertiesConstants.VALUE_PROPERTY_SB);
					    Value value = values[i];
					    if(type.equalsIgnoreCase(PropertiesConstants.TYPE_STRING)) {
					    	valueSB.setAttribute(PropertiesConstants.VALUE_PROPERTY, value.getString());
					    }
					    if(type.equalsIgnoreCase(PropertiesConstants.TYPE_BINARY)) {
					    	valueSB.setAttribute(PropertiesConstants.VALUE_PROPERTY, value.getStream());
					    }
					    if(type.equalsIgnoreCase(PropertiesConstants.TYPE_BOOLEAN)) {
					    	valueSB.setAttribute(PropertiesConstants.VALUE_PROPERTY, new Boolean(value.getBoolean()));
					    }
					    if(type.equalsIgnoreCase(PropertiesConstants.TYPE_DATE)) {
					    	valueSB.setAttribute(PropertiesConstants.VALUE_PROPERTY, value.getDate());
					    }
					    if(type.equalsIgnoreCase(PropertiesConstants.TYPE_LONG)) {
					    	valueSB.setAttribute(PropertiesConstants.VALUE_PROPERTY, new Long(value.getLong()));
					    }
					    if(type.equalsIgnoreCase(PropertiesConstants.TYPE_DOUBLE)) {
					    	valueSB.setAttribute(PropertiesConstants.VALUE_PROPERTY, new Double(value.getDouble()));
					    }
					    propertySB.setAttribute(valueSB);
					}
					propertiesSB.setAttribute(propertySB);
				}
				sourceBean.setAttribute(propertiesSB);
			}
			
			
			
			
			
			// getChilds
			if(incChild && isContainer) {
				Node nodeForChild = node;
				if(requestVer){
					nodeForChild = version;
				}		
				NodeIterator childs = nodeForChild.getNodes();
				SourceBean childsSB = new SourceBean(OperationsConstants.CHILDS);
				while (childs.hasNext()){
					Node child = (Node)childs.next();
					// if node starts with jcr prefix is a system node 
					String nameNode = child.getName();
					if(nameNode.startsWith("jcr:")) {
						continue;
					}
					SourceBean childSB = new SourceBean(OperationsConstants.CHILD);
					childSB.setAttribute(OperationsConstants.NAME, nameNode);
					if(!requestVer) {
						String pathChild = child.getPath();
						childSB.setAttribute(OperationsConstants.PATH, pathChild);
					}
					String uuidChild = "";
					try {
						uuidChild = child.getUUID();
						childSB.setAttribute(OperationsConstants.UUID, uuidChild);
					} catch (Exception e) {	}
					String objectTypeChild = "";
					try {
						 objectTypeChild = child.getProperty(prefixSys + ":" + OperationsConstants.TYPE).getString();
						 childSB.setAttribute(OperationsConstants.TYPE, objectTypeChild);	
					} catch (Exception e) {	}

					childsSB.setAttribute(childSB);
				}
				sourceBean.setAttribute(childsSB);
			}

		    
		    // get content
            if(incCont && !isContainer) { 
            	Node nodeForCont = node;
				if(requestVer){
					nodeForCont = version;
				}
				try {
					InputStream stream = nodeForCont.getProperty(prefixSys + ":" + OperationsConstants.STREAM).getStream();
					sourceBean.setAttribute(OperationsConstants.STREAM, stream);
				} catch (Exception e) {
					sourceBean.setAttribute(OperationsConstants.STREAM, null);
				}
			}
		    
		    
		}
		catch (SourceBeanException e){
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MAJOR,
	    	"GetObjectImpl::toSourceBean: sourceBean error ");
			throw new OperationExecutionException("GetObjectImpl::toSourceBean: sourceBean error", e);
		} catch (PathNotFoundException e) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MINOR,
	    	"GetObjectImpl::toSourceBean: path not find");
			throw new OperationExecutionException("GetObjectImpl::toSourceBean: path not find", e);
		} catch (RepositoryException e) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
	    	"GetObjectImpl::toSourceBean: repository access error");
			throw new OperationExecutionException("GetObjectImpl::toSourceBean: repository access error", e);
		}
		
		return sourceBean;
	}
	
	
}
	
	
	
	
	
	
	
	
	
	
	
	











	
	
	/*
	private SourceBean toSourceBean(Node nodeOr, CMSConnection conn, boolean incChild, Node nodeVer) throws EMFInternalError {
		
		SourceBean sourceBean = null;
		String prefixSys = conn.getPrefixPropertySystem();
		String prefixUsr = conn.getPrefixPropertyUser();
		String objectType = "";
		Node node = nodeOr;
		
		try {
			
			//*************************************************************
			// RECUPERO DATI PRINCIPALI
			//*************************************************************
			
			// popolo il sourcebean con i dati dell'elemento
			sourceBean = new SourceBean(CMSConstants.NODE);
			sourceBean.setAttribute(CMSConstants.NAME, node.getName());
			sourceBean.setAttribute(CMSConstants.PATH, node.getPath());
		    try{
		    	String nameCurVer = node.getBaseVersion().getName();
		    	sourceBean.setAttribute(CMSConstants.VERSION, nameCurVer);
		    } catch(Exception e) {
		    	sourceBean.setAttribute(CMSConstants.VERSION, "");
		    }	
			// le gestioni delle eccezioni seguenti servono in caso di get per la radice, che non possiede i due attributi
			String uuid;
			try{
				uuid = node.getUUID();
			} catch(Exception e) {
				uuid = "";
			}
			sourceBean.setAttribute(CMSConstants.UUID, uuid);
			try {
				objectType = node.getProperty(prefixSys + ":" + CMSConstants.TYPE).getString();
			} catch(Exception e) { }
			sourceBean.setAttribute(CMSConstants.TYPE, objectType);	
			
			//*************************************************************
			// RECUPERO ELENCO DELLE VERSIONI
			//*************************************************************
			
			if(nodeVer==null) {
				SourceBean versionsSB = new SourceBean(CMSConstants.VERSIONS);
				if(node.isNodeType("mix:versionable")) {
					VersionIterator vit = node.getVersionHistory().getAllVersions();
					while (vit.hasNext()){
						SourceBean versionSB = new SourceBean(CMSConstants.VERSION);
						Version v = (Version)vit.next();
						if(!v.getName().equalsIgnoreCase("jcr:rootVersion")) {
							versionSB.setAttribute(CMSConstants.NAME, v.getName());
							versionsSB.setAttribute(versionSB);
						}
					}
					//sourceBean.setAttribute(CMSConstants.VERSION, node.getBaseVersion().getName());
				}
				sourceBean.setAttribute(versionsSB);
			}
			
			
			// se in ingresso vi e un nodo di versione � da quello che devo reperire propriet� e figli
			if(nodeVer != null) {
				node = nodeVer;
			}

			
			//*************************************************************
			// RECUPERO ELENCO DELLE PROPRIETA
			//*************************************************************
			
			PropertyIterator properties = node.getProperties(prefixUsr + ":*");
			SourceBean propertiesSB = new SourceBean(CMSConstants.PROPERTIES);
			while (properties.hasNext()){
				Property property = (Property)properties.next();				
				SourceBean propertySB = new SourceBean(CMSConstants.PROPERTY);
				// ottengo il nome della prop
				String name = property.getName();
				// tolgo il prefisso
				name = name.substring(prefixUsr.length()+1);
				// ottengo il codice del tipo propriet�
				int codType = property.getValue().getType();
				// ottengo il nome del tipo della propriet�
				String type = PropertyType.nameFromValue(codType);
			    //TODO  recuperare il valore della propriet� secondo il tipo	
				// registro il nome, il valore, il codice e il nome del tipo
				propertySB.setAttribute(CMSConstants.NAME, name);
				propertySB.setAttribute(CMSConstants.VALUE, property.getValue().getString());
				propertySB.setAttribute(CMSConstants.CODTYPE, String.valueOf(codType));
				propertySB.setAttribute(CMSConstants.TYPE, type);
				propertiesSB.setAttribute(propertySB);
			}
			sourceBean.setAttribute(propertiesSB);
			
			//*************************************************************
			// RECUPERO ELENCO DEI FIGLI
			//*************************************************************
			
			if(incChild) {
				NodeIterator childs = node.getNodes();
				SourceBean childsSB = new SourceBean(CMSConstants.CHILDS);
				while (childs.hasNext()){
					Node child = (Node)childs.next();
					// se il nodo inizia con il prefisso jcr � un nodo di sistema che non deve essere analizzato
					String nameNode = child.getName();
					if(nameNode.startsWith("jcr:")) {
						continue;
					}
					// ottengo il path del nodo e se l'utentenon ha i permessi di lettura passo al nodo successivo
					String pathChild = child.getPath();
					//  se arrivo a questo punto il nodo deve essere analizzato e visualizzato
					SourceBean childSB = new SourceBean(CMSConstants.CHILD);
					childSB.setAttribute(CMSConstants.PATH, pathChild);
					childSB.setAttribute(CMSConstants.NAME, nameNode);
					
					String uuidChild = "";
					try {
						uuidChild = child.getUUID();
						childSB.setAttribute(CMSConstants.UUID, uuidChild);
					} catch (Exception e) {	}
					String objectTypeChild = "";
					try {
						 objectTypeChild = child.getProperty(prefixSys + ":" + CMSConstants.TYPE).getString();
						 childSB.setAttribute(CMSConstants.TYPE, objectTypeChild);	
					} catch (Exception e) {	}

					childsSB.setAttribute(childSB);
				}
				sourceBean.setAttribute(childsSB);
			}
			
			//*************************************************************
			// RECUPERO EVENTUALE CONTENUTO
			//*************************************************************
						
			// eventuale contenuto di tipo InputStream (binary)
			if (objectType.equalsIgnoreCase(CMSConstants.CONTENT)){
				//SourceBean streamSB = new SourceBean(CMSConstants.CONTENT);
				InputStream stream = node.getProperty(prefixSys + ":" + CMSConstants.STREAM).getStream();
				sourceBean.setAttribute(CMSConstants.STREAM, stream);
				//streamSB.setAttribute(CMSConstants.STREAM, stream);
				//sourceBean.setAttribute(streamSB);
			}
			
			
		} catch (SourceBeanException e){
			System.out.println(e);
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MAJOR,
	    	"GetObjectImpl::toSourceBean: errore nel SourceBean ");
			throw new EMFInternalError(EMFErrorSeverity.ERROR, 
			"GetObjectImpl::toSourceBean: errore nel SourceBean ");
		} catch (PathNotFoundException e) {
			System.out.println(e);
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MINOR,
	    	"GetObjectImpl::toSourceBean: oggetto non trovato");
			throw new EMFInternalError(EMFErrorSeverity.ERROR, 
			"GetObjectImpl::toSourceBean: oggetto non trovato");
		} catch (RepositoryException e) {
			
			System.out.println(e);
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
	    	"GetObjectImpl::toSourceBean: errore in accesso al repository");
			throw new EMFInternalError(EMFErrorSeverity.ERROR, 
			"GetObjectImpl::toSourceBean: errore in accesso al repository");
		}
		return sourceBean;
	}
	
	*/
	
	

	
	
