package it.eng.spago.cms.exec;

import it.eng.spago.cms.exceptions.OperationExecutionException;
import it.eng.spago.cms.exec.operations.Operation;
import it.eng.spago.cms.exec.results.ElementDescriptor;
import it.eng.spago.security.IEngUserProfile;
import it.eng.spago.base.SourceBean;

/**
 * Interface that define methods for interaction with the
 * Content Repository, the possible operations are:
 * <ul>
 * <li>Get information about an object</li>
 * <li>Set a new object</li>
 * <li>Restore an old version of the object</li>
 * <li>Cancel an object</li>
 * <li>Search Objects</li>
 * <li>Rename an object</li>
 * </ul>
 * 
 */


public interface OperationExecutor {
	
	
	/** 
	 * Get information about a node. With opeation parameter is
	 * possible to decide witch inforamtion retrive but if all
	 * inforamtions include child, versions, properties, content
	 * path, uuid, and current version.It's possible to retrive 
	 * inforamtion only of a versin node.
	 */
	public ElementDescriptor getObject(CMSConnection conn, Operation operation, IEngUserProfile profile, boolean closeConn) throws OperationExecutionException;

	/** 
	 *
	 * Set an new object on the repository, if th object doesn't exist 
	 * it will be created otherwise will be created a new version
	 */
	public void setObject(CMSConnection conn, Operation operation, IEngUserProfile profile, boolean closeConn) throws OperationExecutionException;
	
	/** 
	 * Remove a node or one of his versions from the
	 * repository
	 */
	public void deleteObject(CMSConnection conn, Operation operation, IEngUserProfile profile, boolean closeConn) throws OperationExecutionException;
	
	/** 
	 * Restore an old version of a repository object
	 */
	public void restoreObject(CMSConnection conn, Operation operation, IEngUserProfile profile, boolean closeConn) throws OperationExecutionException;
	
	/** 
	 * Perform a search inside the repository executing a 
	 * query expression write with Xpath syntax or with the 
	 * JCR query language
	 */
	public SourceBean search(CMSConnection conn, Operation operation, IEngUserProfile profile, boolean closeConn) throws OperationExecutionException;
	
	/**
	 * Rename an object of the repository
	 */
	public void renameObject(CMSConnection conn, Operation operation, IEngUserProfile profile, boolean closeConn) throws OperationExecutionException;
}
