package it.eng.spago.cms.exec;

import it.eng.javax.jcr.Node;
import it.eng.javax.jcr.NodeIterator;
import it.eng.javax.jcr.PropertyType;
import it.eng.javax.jcr.RepositoryException;
import it.eng.javax.jcr.Session;
import it.eng.javax.jcr.Value;
import it.eng.javax.jcr.query.InvalidQueryException;
import it.eng.javax.jcr.query.Query;
import it.eng.javax.jcr.query.QueryManager;
import it.eng.javax.jcr.query.QueryResult;
import it.eng.javax.jcr.query.Row;
import it.eng.javax.jcr.query.RowIterator;
import it.eng.spago.cms.access.PermissionVerifier;
import it.eng.spago.cms.access.PermissionVerifierManager;
import it.eng.spago.cms.exceptions.OperationExecutionException;
import it.eng.spago.cms.util.OperationDescriptorUtils;
import it.eng.spago.cms.util.constants.CMSConstants;
import it.eng.spago.cms.util.constants.ModuleConstants;
import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.tracing.TracerSingleton;




/**
 * 
 * Implements method for perform the search operation
 * on the repository
 * 
 */
public class SearchObjectImpl {
	
	/**
	 * Implements the search operation on the repository
	 * 
	 * @param parameters, SourceBean of operation configuration
	 * @param conn, Connection to the repository
	 * @return SourceBean with information of the search result
	 * @throws EMFInternalError
	 */
	protected SourceBean execute(SourceBean operation, CMSConnection conn, String [] roles)  throws OperationExecutionException {
		
		SourceBean sourceBean = null;
		Session session = conn.getSession();
		String prefixSys = conn.getPrefixPropertySystem();
		String prefixUsr = conn.getPrefixPropertyUser();
		String queryStr = null;
		String lang = null;
		
		try {
			
			// get path string new object
		    queryStr = OperationDescriptorUtils.getStringAttribute(operation, CMSConstants.QUERY);
			// get the language of the query
		    lang = OperationDescriptorUtils.getStringAttribute(operation, CMSConstants.QUERYLANGUAGE);
		    // create result sourceBean
			sourceBean = new SourceBean(CMSConstants.SEARCHRESULT);
			// Exec Query  
			QueryManager queryManager = session.getWorkspace().getQueryManager();
			Query query = null;
			if(lang.equalsIgnoreCase(CMSConstants.QUERYLANGUAGEJCRQL)) {
				query = queryManager.createQuery(queryStr, Query.SQL );
			} 
			else if(lang.equalsIgnoreCase(CMSConstants.QUERYLANGUAGEXPATH)) {
				query = queryManager.createQuery(queryStr, Query.XPATH);
			}
			// exec query
			QueryResult queryResult = query.execute();
			// retrive information from result			
			NodeIterator ni =  queryResult.getNodes();
			//RowIterator rowIter = queryResult.getRows();
			//String[] properties = queryResult.getPropertyNames();

			
			PermissionVerifier verifier = null;
			try {
				verifier = PermissionVerifierManager.getPermissionVerifier(conn.getNameRepositoryPool());
			} catch (Exception e) {
				throw new OperationExecutionException("SearchObjectImpl::execute: error during the " +
		                "PermissionVerifier recover", e);
			}
			
			SourceBean nodesSB = new SourceBean(CMSConstants.NODES);
			while (ni.hasNext()){
				Node node = ni.nextNode();
                SourceBean nodeSB = new SourceBean(CMSConstants.NODE);
                String uuid = node.getUUID();
                String name = node.getName();
                String path = node.getPath();
                
                if(!verifier.canRead(path, roles)) {
                	continue;
                }
                
                nodeSB.setAttribute(CMSConstants.NAME, name);
                nodeSB.setAttribute(CMSConstants.PATH, path);
                nodeSB.setAttribute(CMSConstants.UUID, uuid);
			    nodesSB.setAttribute(nodeSB);
			}
            sourceBean.setAttribute(nodesSB);
			
			
//			SourceBean rowsSB = new SourceBean(CMSConstants.ROWS);
//			while (rowIter.hasNext()){
//				Row row = rowIter.nextRow();
//				SourceBean rowSB = new SourceBean(CMSConstants.ROW);
//				for(int i=0; i<properties.length; i++) {
//					SourceBean propSB = new SourceBean(CMSConstants.PROPERTY);
//					String prop = properties[i];
//					propSB.setAttribute("name", prop);
//					Value value = row.getValue(prop);
//					int codeType = value.getType();
//					propSB.setAttribute("codeType", String.valueOf(codeType));
//					propSB.setAttribute("nameType", PropertyType.nameFromValue(codeType));
//				    Object val = null;
//					switch(codeType) {
//				    	case 1:
//				    		val = value.getString();
//				    		break;
//				    	case 2:
//				    		val = value.getStream();
//				    		break;
//				    	case 3:
//                            val = new Long(value.getLong()); 
//				    		break;
//				    	case 4:
//				    		val = new Double(value.getDouble());
//				    		break;
//				    	case 5:
//				    		val = value.getDate();
//				    		break;
//				    	case 6:
//				    		val = new Boolean(value.getBoolean());
//				    		break;
//				    	case 7:
//				    		val = value.getString();
//				    		break;
//				    	case 8:
//				    		val = value.getString();
//				    		break;
//				        case 9:
//				        	val = value.getString();
//				        	break;
//				    }
//					propSB.setAttribute("value", val);
//				    rowSB.setAttribute(propSB);
//				}
//				rowsSB.setAttribute(rowSB);
//			}
//			sourceBean.setAttribute(rowsSB);

			
			
			
		} catch (InvalidQueryException e) {
			TracerSingleton.log(ModuleConstants.NAME_MODULE, TracerSingleton.MAJOR,
	    	"SearchObjectImpl::execute: query not valid");				
		    throw new OperationExecutionException("SearchObjectImpl::execute: query not valid");			
		} catch (RepositoryException e) {
			TracerSingleton.log(ModuleConstants.NAME_MODULE, TracerSingleton.CRITICAL,
	    	"SearchObjectImpl::execute: repository access error");
			throw new OperationExecutionException("SearchObjectImpl::execute: repository access error");
		} catch (SourceBeanException e) {
			TracerSingleton.log(ModuleConstants.NAME_MODULE, TracerSingleton.MAJOR,
	    	"SearchObjectImpl::toSourceBean: SourceBean error ");
			throw new OperationExecutionException("SearchObjectImpl::toSourceBean: SourceBean error");			
		}
		return sourceBean;
	}	
	
}
