package it.eng.spago.cms.exec.operations;

import it.eng.spago.base.SourceBean;
import it.eng.spago.cms.exceptions.BuildOperationException;
import it.eng.spago.cms.util.constants.ModuleConstants;
import it.eng.spago.cms.util.constants.OperationsConstants;
import it.eng.spago.tracing.TracerSingleton;

/*
	<OPERATION name="">
		<DELETEOPERATION path="?" version="?" />
	</OPERATION> 
*/


public class DeleteOperation implements Operation {

	private SourceBean operationDescriptor = null;
	
	public DeleteOperation() throws BuildOperationException{	
		try {
			operationDescriptor = new SourceBean(OperationsConstants.NAME_DELETE_OPERATION);
			operationDescriptor.setAttribute(OperationsConstants.PATH, "");
			operationDescriptor.setAttribute(OperationsConstants.VERSION, "");
		} catch (Exception e) {
			TracerSingleton.log(ModuleConstants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "DeleteOperation::<init>(): Error during the creation of the delete operation" +
                                "SourceBean");
			throw new BuildOperationException("DeleteOperation::<init>(): Error during the creation of the " +
					                          "delete operation SourceBean", e);
		}
	}
	
	public DeleteOperation(String path) throws BuildOperationException{	
		try {
			operationDescriptor = new SourceBean(OperationsConstants.NAME_DELETE_OPERATION);
			operationDescriptor.setAttribute(OperationsConstants.PATH, path);
			operationDescriptor.setAttribute(OperationsConstants.VERSION, "");
		} catch (Exception e) {
			TracerSingleton.log(ModuleConstants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "DeleteOperation::<init>(String): Error during the creation of the delete operation" +
                                "SourceBean");
			throw new BuildOperationException("DeleteOperation::<init>(String): Error during the creation of the " +
					                          "delete operation SourceBean", e);
		}
	}
	
	public DeleteOperation(String path, String version) throws BuildOperationException{	
		try {
			operationDescriptor = new SourceBean(OperationsConstants.NAME_DELETE_OPERATION);
			operationDescriptor.setAttribute(OperationsConstants.PATH, path);
			operationDescriptor.setAttribute(OperationsConstants.VERSION, version);
		} catch (Exception e) {
			TracerSingleton.log(ModuleConstants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "DeleteOperation::<init>(String, String): Error during the creation of the delete operation" +
                                "SourceBean");
			throw new BuildOperationException("DeleteOperation::<init>(String, String): Error during the creation of the " +
					                          "delete operation SourceBean", e);
		}
	}
	
	public DeleteOperation(SourceBean descriptor){	
		operationDescriptor = descriptor;
	}
	
	public SourceBean getOperationDescriptor() {
		return operationDescriptor;
	}

	public boolean isComplete() {
		if(operationDescriptor==null) return false;
		String path = (String)operationDescriptor.getAttribute(OperationsConstants.PATH);
		if(path==null) return false;
		if(path.trim().equals("")) return false;
		return true;
	}

	
	public void setPath(String path) throws BuildOperationException {
		try {
			operationDescriptor.updAttribute(OperationsConstants.PATH, path);
		} catch (Exception e) {
			TracerSingleton.log(ModuleConstants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "DeleteOperation::setPath: Error during the path setting");
            throw new BuildOperationException("DeleteOperation::setPath: Error during the path setting", e);
		}
	}
	
	public void setVersion(String version) throws BuildOperationException {
		try {
			operationDescriptor.updAttribute(OperationsConstants.VERSION, version);
		} catch (Exception e) {
			TracerSingleton.log(ModuleConstants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "DeleteOperation::setVersion: Error during the version setting");
            throw new BuildOperationException("DeleteOperation::setVersion: Error during the version setting", e);
		}
	}
	
	
}
